{-# LANGUAGE OverloadedStrings #-}
--------------------------------------------------------------------
-- |
-- Copyright :  (c) Stephen Diehl 2013
-- License   :  MIT
-- Maintainer:  stephen.m.diehl@gmail.com
-- Stability :  experimental
-- Portability: non-portable
--
--------------------------------------------------------------------

module Main where

{-# LANGUAGE OverloadedStrings #-}
import           Hakyll
import           Text.Pandoc as Pandoc
import qualified Data.Text as T
import qualified System.Process  as Process
import           System.FilePath (replaceExtension, takeDirectory)
import           Data.Maybe (isJust)
import           Hakyll.Images ( loadImage
                                , scaleImageCompiler
                               )
import           System.Exit     (ExitCode)
-- import           Data.List        (isPrefixOf, isSuffixOf)
-- import           System.FilePath  (isAbsolute, normalise, takeFileName, makeRelative)

--------------------------------------------------------------------------------
-- | Entry point
main :: IO ()
main = hakyllWith config $ do
    -- Resize images
    match "img/thumbnails/**.png" $ do
        route idRoute
        compile $ loadImage
            >>= scaleImageCompiler 140 140

    match "img/heads/**.png" $ do
        route idRoute
        compile $ loadImage
            >>= scaleImageCompiler 256 256

    match "img/large/**.png" $ do
        route idRoute
        compile $ loadImage
            >>= scaleImageCompiler 900 262

    -- copying stuff
    match ("fonts/*"
        .||. "img/*"
        .||. "img/*/**.png"
        .||. "css/*"
        .||. "js/*"
        .||. "reveal.js/dist/**"
        .||. "reveal.js/plugin/**"
        .||. "cours/prog_seq/slides/figs/*"
        -- .||. "cours/algo/slides/figs/*"
        .||. "cours/math_tech_info/figs/*"
        -- .||. "cours/algo/slides/*.json"
        .||. "cours/algo/slides/*.pdf"
        .||. "cours/math_tech_info/cours.pdf"
        .||. "cours/isc_physics/cours.pdf"
        .||. "cours/isc_physics/figs/*") $ do
      route idRoute
      compile $ copyFileCompiler

    -- Render the 404 page, we don't relativize URL's here.
    match "404.html" $ do
        route idRoute
        compile $ pandocCompiler
            >>= loadAndApplyTemplate "templates/page.html" postCtx

    -- Phys app posts
    match "cours/isc_physics/*.markdown" $ do
        route $ setExtension "html"
        compile $ pandocCrossrefNumberingCompiler
            >>= loadAndApplyTemplate "templates/class.html" postCtx
            >>= relativizeUrls

    -- Phys app post list
    create ["cours/phys_app.html"] $ do
        route idRoute
        compile $ do
            posts <- recentFirst =<< loadAll ("cours/isc_physics/*.markdown" .&&. hasNoVersion)
            makeItem ""
                >>= loadAndApplyTemplate "templates/course.html" (courseCtx posts "Physique appliquée" "cours/isc_physics/cours.pdf")
                >>= relativizeUrls

    -- Math Tech Info posts
    match "cours/math_tech_info/*.markdown" $ do
        route $ setExtension "html"
        compile $ pandocCrossrefNumberingCompiler
            >>= loadAndApplyTemplate "templates/class.html" postCtx
            >>= relativizeUrls

    -- Math Tech Info posts in PDF. Producing the .pdf
    -- PDF produced but putting it into a list not...
    -- match "cours/math_tech_info/1_Rappel.markdown" $ version "pdf" $ do
    --     route   $ setExtension "pdf"
    --     compile $ do getResourceBody
    --         >>= readPandoc
    --         >>= writeXeTex
    --         >>= loadAndApplyTemplate "templates/default.latex" defaultContext
    --         >>= xelatex

    -- Math Tech Info post list
    create ["cours/math_tech_info.html"] $ do
        route idRoute
        compile $ do
            posts <- recentFirst =<< loadAll ("cours/math_tech_info/*.markdown" .&&. hasNoVersion)
            makeItem ""
                >>= loadAndApplyTemplate "templates/course.html" (courseCtx posts "Mathématiques en technologie de l'information" "cours/math_tech_info/cours.pdf")
                >>= relativizeUrls

    -- Programmation séquentielle slides
    match "cours/prog_seq/slides/*.markdown" $ do
        route $ setExtension "html"
        compile $ pandocRevealCompiler
            >>= loadAndApplyTemplate "templates/reveal.html" postCtx
            >>= relativizeUrls

    -- Prog seq post list
    create ["cours/prog_seq.html"] $ do
        route idRoute
        compile $ do
            posts <- recentFirst =<< loadAll "cours/prog_seq/slides/*.markdown"
            makeItem ""
                >>= loadAndApplyTemplate "templates/archive.html" (pagesCtx posts "Programmation séquentielle")
                >>= relativizeUrls


    -- Algorithmique slides
    -- match "cours/algo/slides/*.markdown" $ do
    --     route $ setExtension "html"
    --     compile $ pandocRevealCompiler
    --         >>= loadAndApplyTemplate "templates/reveal.html" postCtx
    --         >>= relativizeUrls

    match "cours/algo/slides/index.md" $ do
        route $ setExtension "html"
        compile $ pandocCompiler
            >>= loadAndApplyTemplate "templates/post.html" postCtx
            >>= relativizeUrls   

    -- Prog seq post list
    create ["cours/algo.html"] $ do
        route idRoute
        compile $ do
            posts <- recentFirst =<< loadAll "cours/algo/slides/index.md"
            makeItem ""
                >>= loadAndApplyTemplate "templates/archive.html" (courseCtx posts "Les slides d'Algorithmique en PDF" "algo/slides/index.html")
                >>= relativizeUrls

    -- Team description
    match "team/*.md" $ do
        route $ setExtension "html"
        compile $ pandocCompiler
            >>= relativizeUrls

    -- Team members list
    create ["pages/team.html"] $ do
        route idRoute
        compile $ do
            posts <- recentFirst =<< loadAll "team/*.md"
            makeItem ""
                >>= loadAndApplyTemplate "templates/team.html" (pagesCtx posts "Research group")
                >>= relativizeUrls

    -- Research description
    match "posts/research/*.md" $ do
        route $ setExtension "html"
        compile $ pandocCompiler
            >>= loadAndApplyTemplate "templates/post.html" postCtx
            >>= relativizeUrls

    -- Research topics
    create ["pages/research_projects.html"] $ do
        route idRoute
        compile $ do
            posts <- recentFirst =<< loadAll "posts/research/*.md"
            makeItem ""
                >>= loadAndApplyTemplate "templates/archive.html" (pagesCtx posts "Research projects")
                >>= relativizeUrls

    -- Team description
    match "posts/bachelor/*.md" $ do
        route $ setExtension "html"
        compile $ pandocCompiler
            >>= loadAndApplyTemplate "templates/post.html" postCtx
            >>= relativizeUrls

    -- Team members list
    create ["pages/bachelor_projects.html"] $ do
        route idRoute
        compile $ do
            posts <- recentFirst =<< loadAll "posts/bachelor/*.md"
            makeItem ""
                >>= loadAndApplyTemplate "templates/archive.html" (pagesCtx posts "Bachelor projects")
                >>= relativizeUrls

    -- Render some static pages
    match (fromList staticPages) $ do
        route   $ setExtension ".html"
        compile $ pandocCompiler
            >>= loadAndApplyTemplate "templates/page.html" postCtx
            >>= relativizeUrls

    -- Index
    match "index.html" $ do
        route idRoute
        compile $ getResourceBody
            >>= applyAsTemplate defaultContext
            >>= relativizeUrls

    -- match "pages/*" $ do
    --     route $ setExtension "html"
    --     compile $ getResourceBody
    --         >>= loadAndApplyTemplate "templates/page.html"    postCtx
    --         >>= relativizeUrls



    -- Read templates
    match "templates/*" $ compile templateCompiler

    where
        staticPages =
            [ "pages/contact.md"
            -- , "cours/algo/slides/index.md"
            ]

    -- Used to compile to PDF
    -- where
    --   writeXeTex :: Item Pandoc.Pandoc -> Compiler (Item String)
    --   writeXeTex = traverse $ \pandoc ->
    --       case Pandoc.runPure (Pandoc.writeLaTeX Pandoc.def pandoc) of
    --           Left err -> fail $ show err
    --           Right x  -> return (T.unpack x)

--------------------------------------------------------------------
-- Contexts
--------------------------------------------------------------------

postCtx :: Context String
postCtx =
  dateField "date" "%B %e, %Y"
  `mappend` mathCtx
  `mappend` defaultContext

mathCtx :: Context String
mathCtx = field "mathjax" $ \item -> do
  metadata <- getMetadata $ itemIdentifier item
  return ""
  return $ if isJust $ lookupString "mathjax" metadata
           then "<script type=\"text/javascript\" src=\"https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.0/MathJax.js?config=TeX-AMS_HTML\"></script>"
           else ""

noPostCtx title pdfurl =
  constField "title" title
  `mappend` constField "pdfurl" pdfurl
  `mappend` defaultContext

courseCtx posts title pdfurl =
  listField "posts" postCtx (return posts)
  `mappend` constField "title" title
  `mappend` constField "pdfurl" pdfurl
  `mappend` defaultContext

pagesCtx posts title =
  listField "posts" postCtx (return posts)
  `mappend` constField "title" title
  `mappend` defaultContext

--------------------------------------------------------------------
-- Configuration
--------------------------------------------------------------------

myPandocCompiler :: Compiler (Item String)
myPandocCompiler = pandocCompilerWith defaultHakyllReaderOptions pandocOptions 

pandocOptions :: WriterOptions
pandocOptions = defaultHakyllWriterOptions
    { 
        writerExtensions = defaultPandocExtensions
        , writerHTMLMathMethod = MathJax ""
        , writerNumberSections = True 
        , writerTableOfContents = True
    }

-- Pandoc extensions used by the myPandocCompiler
defaultPandocExtensions :: Extensions
defaultPandocExtensions = 
    let extensions = [ 
            -- Pandoc Extensions: http://pandoc.org/MANUAL.html#extensions
            -- Math extensions
              Ext_tex_math_dollars
            , Ext_tex_math_double_backslash
            , Ext_latex_macros
                -- Code extensions
            , Ext_fenced_code_blocks
            , Ext_backtick_code_blocks
            , Ext_fenced_code_attributes
            , Ext_inline_code_attributes        -- Inline code attributes (e.g. `<$>`{.haskell})
                -- Markdown extensions
            , Ext_implicit_header_references    -- We also allow implicit header references (instead of inserting <a> tags)
            , Ext_definition_lists              -- Definition lists based on PHP Markdown
            , Ext_yaml_metadata_block           -- Allow metadata to be speficied by YAML syntax
            , Ext_superscript                   -- Superscripts (2^10^ is 1024)
            , Ext_subscript                     -- Subscripts (H~2~O is water)
            , Ext_footnotes                     -- Footnotes ([^1]: Here is a footnote)
            ]
        defaultExtensions = writerExtensions defaultHakyllWriterOptions

    in foldr enableExtension defaultExtensions extensions

pandocCrossrefNumberingCompiler :: Compiler (Item String)
pandocCrossrefNumberingCompiler = do 
    getResourceBody 
        >>= withItemBody (unixFilter "pandoc" ["-F"
                                            , "pandoc-numbering"
                                            , "-F"
                                            , "mermaid-filter"
                                            , "-F"
                                            , "pandoc-crossref"
                                            , "-t"
                                            , "markdown"
                                            ])
        >>= readPandocWith defaultHakyllReaderOptions
        >>= return . writePandocWith pandocOptions



pandocRevealCompiler :: Compiler (Item String)
pandocRevealCompiler = do 
    getResourceBody 
        >>= withItemBody (unixFilter "pandoc" ["-F"
                                            , "pandoc-numbering"
                                            , "-F"
                                            , "mermaid-filter"
                                            , "-F"
                                            , "pandoc-crossref"
                                            , "-t"
                                            , "revealjs"
                                            , "-V"
                                            , "revealjs-url=./reveal.js"
                                            , "-V"
                                            , "theme=white"
                                            ])
        >>= readPandocWith defaultHakyllReaderOptions
        >>= return . writePandocWith pandocOptions

--------------------------------------------------------------------------------
-- | Hacky.
-- xelatex :: Item String -> Compiler (Item TmpFile)
-- xelatex item = do
--     TmpFile texPath <- newTmpFile "xelatex.tex"
--     let tmpDir  = takeDirectory texPath
--         pdfPath = replaceExtension texPath "pdf"

--     unsafeCompiler $ do
--         writeFile texPath $ itemBody item
--         _ <- Process.system $ unwords ["xelatex", "-halt-on-error",
--             "-output-directory", tmpDir, texPath, ">/dev/null", "2>&1"]
--         return ()

--     makeItem $ TmpFile pdfPath


--------------------------------------------------------------------------------
config :: Configuration
config = defaultConfiguration
    {
          deploySite = deploy
        -- , ignoreFile = ignoreFile'
    }
  where
  --   ignoreFile' path
  --       | "."    `isPrefixOf` fileName = False
  --       | "#"    `isPrefixOf` fileName = True
  --       | "~"    `isSuffixOf` fileName = True
  --       | ".swp" `isSuffixOf` fileName = True
  --       | otherwise                    = False
  --     where
  --       fileName = takeFileName path
    deploy :: Configuration -> IO ExitCode
    deploy _c = do
        Process.rawSystem "rsync"
                [ "--checksum", "-avzz"
                , "_site/", "ur1bg_malas@ur1bg.ftp.infomaniak.com:web/malaspinas/beta/"
                ]
